@@ -1,11 +1,9 @@
 // ---------------------------------------------------------------------
-// $Id: DiskUsage.cc 156 2008-06-08 07:13:49Z daaugusto $
-//
 //   DiskUsage.cc (created on Tue Aug 23 01:08:35 BRT 2005)
 // 
 //   Genetic Algorithm File Fitter (gaffitter)
 //
-//   Copyright (C) 2005-2008 Douglas A. Augusto
+//   Copyright (C) 2005-2015 Douglas A. Augusto
 // 
 // This file is part of gaffitter.
 // 
@@ -38,12 +36,18 @@
 */
 #define _FILE_OFFSET_BITS 64
 
+#ifndef __MINGW32__
+#define HAVE_LSTAT
+#endif
+
 #include "DiskUsage.h"
 #include "util/Exception.h"
 
 #include <sys/stat.h>
 #include <dirent.h>
 #include <unistd.h>
+#include <cstring> // for strerror
+#include <errno.h> // for errno
 
 #include <string>
 #include <iostream>
@@ -55,68 +59,75 @@ unsigned DiskUsage::m_block_size;
 
 //----------------------------------------------------------------------
 Params::UBigInt 
-DiskUsage::GetSize(const char* filename) 
+DiskUsage::GetSize( const char* filename ) 
 {
    struct stat st;
-   if (lstat(filename, &st) != 0)
+
+#ifdef HAVE_LSTAT
+   if( lstat( filename, &st ) != 0 )
+#else
+   if( stat( filename, &st ) != 0 )
+#endif
    {
       cerr << "> Could not read: " << filename << endl;
       return 0;
    }
       
-   if (S_ISDIR(st.st_mode))
+   if( S_ISDIR( st.st_mode ) )
    {
       char saved_path[m_max_path_name], tmp_path[m_max_path_name];
 
       // saves current path before chdir to 'filename' directory
-      if (getcwd(saved_path, m_max_path_name-1) == NULL)
+      if( getcwd( saved_path, m_max_path_name - 1 ) == NULL )
       {
-         cerr << "> Path name too long; maximum " << m_max_path_name-1 
-              << " chars." << endl;
+         cerr << "> Could not get current dir: " << strerror( errno ) << endl;
          return 0;
       }
 
-      if (chdir(filename) != 0) 
+      if( chdir( filename ) != 0 ) 
       {
          cerr << "> Could not chdir to: " << filename << endl;
          return 0;
       }
 
-      if (getcwd(tmp_path, m_max_path_name-1) == NULL)
+      if( getcwd( tmp_path, m_max_path_name - 1 ) == NULL )
       {
-         cerr << "> Path name too long; maximum " << m_max_path_name-1 
-              << " chars." << endl;
+         cerr << "> Could not get current dir: " << strerror( errno ) << endl;
          return 0;
       }
 
-      if (lstat(tmp_path, &st) != 0)
+#ifdef HAVE_LSTAT
+      if( lstat( tmp_path, &st ) != 0 )
+#else
+      if( stat( tmp_path, &st ) != 0 )
+#endif
       {
          cerr << "> Could not read dir: " << tmp_path << endl;
          return 0;
       }
 
       Params::UBigInt total = 
-                AllocationSize(static_cast<Params::UBigInt>(st.st_size)) 
-                                         + DepthFirstTraversal(tmp_path);
+                AllocationSize( static_cast<Params::UBigInt>( st.st_size ) ) 
+                                         + DepthFirstTraversal( tmp_path );
 
       // return to the original path (important!)
-      chdir(saved_path);
+      chdir( saved_path );
 
       return total;
    }
    else // ISREG, ISLNK, ISFIFO, ISBLK, ...
-      return AllocationSize(static_cast<Params::UBigInt>(st.st_size));
+      return AllocationSize( static_cast<Params::UBigInt>( st.st_size ) );
 }
 
 //----------------------------------------------------------------------
 Params::UBigInt
-DiskUsage::AllocationSize(Params::UBigInt size) 
+DiskUsage::AllocationSize( Params::UBigInt size ) 
 {
    Params::UBigInt remainder = size % m_block_size;
 
    /* Test whether the filesize is smaller than the allocation block. 
     Also, 0-sized files occupy space equal to the allocation block size.  */
-   if (remainder || size==0) 
+   if( remainder || size==0 ) 
       return size += m_block_size - remainder;
    
    // the filesize matches perfectly the allocation block
@@ -125,9 +136,9 @@ DiskUsage::AllocationSize(Params::UBigInt size)
 
 //----------------------------------------------------------------------
 Params::UBigInt
-DiskUsage::DepthFirstTraversal(const char* currdir) 
+DiskUsage::DepthFirstTraversal( const char* currdir ) 
 {
-   if (chdir(currdir) != 0)
+   if( chdir( currdir ) != 0 )
    {
       cerr << "> Could not chdir to: " << currdir << endl;
       return 0;
@@ -136,42 +147,46 @@ DiskUsage::DepthFirstTraversal(const char* currdir)
    Params::UBigInt sum = 0;
 
    // open the directory
-   DIR *dir = opendir(".");
-   if (dir == NULL)
+   DIR *dir = opendir( "." );
+   if( dir == NULL )
    {
       cerr << "> Could not open dir: " << currdir << endl;
-      chdir("..");
+      chdir( ".." );
 
       return 0;
    }
 
    struct dirent *de;
-   while ((de = readdir(dir)))
+   while( ( de = readdir( dir ) ) )
    {
       struct stat st;
-      if (lstat(de->d_name, &st) != 0)
+#ifdef HAVE_LSTAT
+      if( lstat( de->d_name, &st ) != 0 )
+#else
+      if( stat( de->d_name, &st ) != 0 )
+#endif
       {
          cerr << "> Could not read: " << de->d_name << endl;
          continue;
       }
 
-      if (S_ISDIR(st.st_mode)) // is a directory (= recursive)
+      if( S_ISDIR( st.st_mode ) ) // is a directory (= recursive)
       {
          string tmp_str = de->d_name;
 
-         if ((tmp_str != ".") && (tmp_str != ".."))
+         if( ( tmp_str != "." ) && ( tmp_str != ".." ) )
          {
-            string filename = string(currdir) + "/" + tmp_str;
+            string filename = string( currdir ) + "/" + tmp_str;
 
-            sum += AllocationSize(static_cast<Params::UBigInt>(st.st_size)) 
-                                    + DepthFirstTraversal(filename.c_str());
+            sum += AllocationSize( static_cast<Params::UBigInt>( st.st_size ) ) 
+                                    + DepthFirstTraversal( filename.c_str() );
          }
       }
       else // ISREG, ISLNK, ISFIFO, ISBLK, ...
-         sum += AllocationSize(static_cast<Params::UBigInt>(st.st_size));
+         sum += AllocationSize( static_cast<Params::UBigInt>( st.st_size ) );
    }
-   closedir(dir);
-   chdir("..");
+   closedir( dir );
+   chdir( ".." );
 
    return sum;
 }
