@@ -1,11 +1,9 @@
 // ---------------------------------------------------------------------
-// $Id: Params.cc 235 2008-08-15 20:10:08Z daaugusto $
-//
 //   Params.cc (created on Tue Aug 23 18:48:35 BRT 2005)
 // 
 //   Genetic Algorithm File Fitter (gaffitter)
 //
-//   Copyright (C) 2005-2008 Douglas A. Augusto
+//   Copyright (C) 2005-2015 Douglas A. Augusto
 // 
 // This file is part of gaffitter.
 // 
@@ -39,18 +37,18 @@ Params::ShowVersion() const // --version
 {
    cout
 
-   << "gaffitter 0.6.0 (Genetic Algorithm File Fitter) (C) 2005-2008\n"
+   << "gaffitter 1.0.0 (Genetic Algorithm File Fitter) (C) 2005-2015\n"
    << "\n"
    << "This is free software. You may redistribute copies of it under the terms\n"
    << "of the GNU General Public License <http://www.gnu.org/licenses/gpl.html>.\n"
    << "There is NO WARRANTY, to the extent permitted by law.\n"
    << "\n"
-   << "Written by Douglas Adriano Augusto (daaugusto).\n";
+   << "Written by Douglas A. Augusto (daaugusto).\n";
 }
 
 //---------------------------------------------------------------------
 void
-Params::ShowUsage(const char* app = "gaffitter") const // -h or --help
+Params::ShowUsage(const char* app = "fit") const // -h or --help
 {
    cout  
 
@@ -104,6 +102,8 @@ Params::ShowUsage(const char* app = "gaffitter") const // -h or --help
    << "     enclose file names with \"char\" [default = none]\n"
    << "  --dw <char>, --delimit-with <char>\n"
    << "     delimit file names (lines) with \"char\" [default = newline]\n"
+   << "  -1 (or --fast) ... -9 (or --best)\n"
+   << "     select preset search parameters [default = -3]\n"
    << "  --version\n"
    << "     print GAFFitter version and exit\n"
    << "  -h, --help\n"
@@ -136,9 +136,9 @@ Params::ShowUsage(const char* app = "gaffitter") const // -h or --help
    << "  --ga-ps <n>, --ga-pop-size <n>\n"
    << "     number of individuals, n>tournament_size [default = auto]\n"
    << "  --ga-cp <f>, --ga-cross-prob <f>\n"
-   << "     crossover probability, 0.0<=f<=1.0 [default = 0.95]\n"
+   << "     crossover probability, 0.0<=f<=1.0 [default = 0.75]\n"
    << "  --ga-mp <f>, --ga-mutation-prob <f>\n"
-   << "     mutation probability, 0.0<=f<=1.0 [default = 0.10]\n"
+   << "     mutation probability, 0.0<=f<=1.0 [default = 0.30]\n"
    << "  --ga-sp <n>, --ga-sel-pressure <n>\n"
    << "     selection pressure (tournament size), 2<=n<pop_size [default = 2]\n"
    << "  --ga-theo [n], --ga-theoretical [n]\n"
@@ -206,8 +206,8 @@ Params::Initialize()
 
    O.Int.Add("--ga-ng","--ga-num-generations",0,0,numeric_limits<int>::max());
    O.Int.Add("--ga-ps","--ga-pop-size",0,0,numeric_limits<int>::max());
-   O.Float.Add("--ga-cp","--ga-cross-prob",0.95,0.0,1.0);
-   O.Float.Add("--ga-mp","--ga-mutation-prob",0.10,0.0,1.0);
+   O.Float.Add("--ga-cp","--ga-cross-prob",0.75,0.0,1.0);
+   O.Float.Add("--ga-mp","--ga-mutation-prob",0.30,0.0,1.0);
    O.Int.Add("--ga-sp","--ga-sel-pressure",0,0,numeric_limits<int>::max());
    O.Bool.Add("--ga-rs","--ga-random-seed");
    O.Int.Add("--ga-s","--ga-seed",1,0,numeric_limits<long>::max());
@@ -228,6 +228,17 @@ Params::Initialize()
    O.Char.Add("--ew","--enclose-with",'\0');
    O.Char.Add("--bs","--bins-separator",'\n');
 
+   // Presets
+   O.Bool.Add("-1","--fast");
+   O.Bool.Add("-2");
+   O.Bool.Add("-3");
+   O.Bool.Add("-4");
+   O.Bool.Add("-5");
+   O.Bool.Add("-6");
+   O.Bool.Add("-7");
+   O.Bool.Add("-8");
+   O.Bool.Add("-9","--best");
+
    // -- Get the options! ----------------
    /* Right now, the 'O' object will process the command-line, i.e.,
     * it will try to recognize the options and their respective arguments. */
@@ -237,7 +248,7 @@ Params::Initialize()
    if (O.Bool.Get("-h")) { ShowUsage(); return false; /* exit */ }
    if (O.Bool.Get("--version")) { ShowVersion(); return false; /* exit */ }
 
-   // input via PIPE: ... | gaffitter - ...
+   // input via PIPE: ... | fit - ...
    m_pipe = O.Bool.Get("-");
 
    // ---- units
@@ -321,7 +332,40 @@ Params::Initialize()
 
    // ---- Genetic Algorithm ------------------------------------------
    /* m_ga_pop_size and m_ga_num_gens are (auto)adjusted in
-    * Optimizer::Initialize */
+    * Optimizer::Initialize depending on the input size */
+
+   // -- Presets
+   if (O.Bool.Get("-9")) {
+      m_ga_psf = 95; m_ga_ngf = 85; m_ga_cross_prob = 0.95; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-8")) {
+      m_ga_psf = 90; m_ga_ngf = 55; m_ga_cross_prob = 0.80; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-7")) {
+      m_ga_psf = 95; m_ga_ngf = 45; m_ga_cross_prob = 0.75; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-6")) {
+      m_ga_psf = 60; m_ga_ngf = 40; m_ga_cross_prob = 0.75; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-5")) {
+      m_ga_psf = 55; m_ga_ngf = 40; m_ga_cross_prob = 0.80; m_ga_mut_prob = 0.45;
+   }
+   else if (O.Bool.Get("-4")) {
+      m_ga_psf = 50; m_ga_ngf = 35; m_ga_cross_prob = 0.85; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-3")) {
+      m_ga_psf = 30; m_ga_ngf = 40; m_ga_cross_prob = 0.75; m_ga_mut_prob = 0.30;
+   }
+   else if (O.Bool.Get("-2")) {
+      m_ga_psf = 10; m_ga_ngf = 75; m_ga_cross_prob = 1.00; m_ga_mut_prob = 0.50;
+   }
+   else if (O.Bool.Get("-1")) {
+      m_ga_psf = 5; m_ga_ngf = 65; m_ga_cross_prob = 0.90; m_ga_mut_prob = 0.50;
+   }
+   else {
+      // Default preset's values (default preset = -3)
+      m_ga_psf = 30; m_ga_ngf = 40; m_ga_cross_prob = 0.75; m_ga_mut_prob = 0.30;
+   }
 
    // -- Number of generations
    m_ga_num_gens = O.Int.Get("--ga-ng"); // check range: done in 
@@ -330,12 +374,11 @@ Params::Initialize()
    m_ga_pop_size = O.Int.Get("--ga-ps"); // check range: done in 
                                             // Optimizer::Initialize
 
-   // -- Crossover probability
-   m_ga_cross_prob = O.Float.Get("--ga-cp");
+   // -- Crossover probability (the user may override preset values)
+   if (O.Float.Found("--ga-cp")) m_ga_cross_prob = O.Float.Get("--ga-cp");
 
-   // -- Mutation probability
-   m_ga_mut_prob = O.Float.Get("--ga-mp"); // automatic value adjusted 
-                                              // by GeneticAlgorithm (~1/L)
+   // -- Mutation probability (the user may override preset values)
+   if (O.Float.Found("--ga-mp")) m_ga_mut_prob = O.Float.Get("--ga-mp");
 
    // -- Selection pressure (currently "tournament size")
    m_ga_sel_pressure = O.Int.Get("--ga-sp"); // check range: done in 
@@ -345,7 +388,7 @@ Params::Initialize()
    if (O.Bool.Get("--ga-rs")) m_ga_seed = 0;
    else m_ga_seed = O.Int.Get("--ga-s");
 
-   // -- stop the search when the theorical minimum number of bins is
+   // -- stop the search when the theoretical minimum number of bins is
    // reached?
    m_bins_theo = O.Int.Get( "--ga-theo" );
    m_theoretical = O.Bool.Get( "--ga-theo" ) || m_bins_theo > 0;
