@@ -1,11 +1,9 @@
 // ---------------------------------------------------------------------------
-// $Id: GeneticAlgorithm.cc 231 2008-08-14 04:51:19Z daaugusto $
-//
 //   GeneticAlgorithm.cc (created on Tue Nov 08 01:08:35 BRT 2005)
 // 
 //   Genetic Algorithm File Fitter (gaffitter)
 //
-//   Copyright (C) 2005-2008 Douglas A. Augusto
+//   Copyright (C) 2005-2015 Douglas A. Augusto
 // 
 // This file is part of gaffitter.
 // 
@@ -64,16 +62,16 @@ GeneticAlgorithm::GeneticAlgorithm( Input& i ): Optimizer( i ), m_cur_gen( 0 )
       Tournament = &GeneticAlgorithm::TournamentN;
    }
 
-   // Population size and generations are proportionally to ln (natural
+   // Population size and generations are proportional to ln (natural
    // logarithmic) of input size (number of files).
    if( m_params.m_ga_pop_size <= m_tournament_size ) 
        m_params.m_ga_pop_size = 
             max( m_tournament_size + 1, 
-            static_cast<unsigned>( 20 * log( (float) m_files.size() + 1 ) ) );
+            static_cast<unsigned>( m_params.m_ga_psf * log( (float) m_files.size() + 1 ) ) );
     
    if( m_params.m_ga_num_gens <= 0 ) 
        m_params.m_ga_num_gens = 
-              static_cast<unsigned>( 50 * log( (float) m_files.size() + 1 ) );
+              static_cast<unsigned>( m_params.m_ga_ngf * log( (float) m_files.size() + 1 ) );
 
    // reserve the correct size of the population
    m_individuals.reserve( m_params.m_ga_pop_size );
@@ -177,14 +175,19 @@ GeneticAlgorithm::InitPopulation()
 {
    ++m_cur_gen; // the initial population is the first generation
 
+   // Using a particular random function [0,N) in order to share the seed
+   // (state). The indication of the overloaded function is needed:
+   unsigned long (*rand_func)( unsigned long ) = &Random::Int;
+
    for( unsigned j = 0; j < m_params.m_ga_pop_size; ++j )
    {
       // Insert a new individual in population (m_individuals)
       m_individuals.push_back( Individual() );
 
-      // Generate a random permutation of the input items/files
-      for( unsigned i = 0; i < m_files.size(); ++i ) 
-         swap( m_files[i], m_files[Random::Int( 0, m_files.size() - 1 )] );
+      // Generate a random permutation of the input items/files. The STL's
+      // random_shuffle uses the Durstenfeld's algorithm (clever implementation
+      // of Fisher-Yates shuffle), so it is unbiased.
+      std::random_shuffle( m_files.begin(), m_files.end(), rand_func );
 
       // Add the above permutation via FirstFit algorithm
       FirstFit( m_individuals.back().m_genome, m_files );
@@ -681,7 +684,7 @@ GeneticAlgorithm::DominanceForThree( Bin& bin,
 
                         break;
 #else
-                        return 1; // juste one improvement
+                        return 1; // just one improvement
 #endif
                      }
                   } else break; // unassigned is sorted
