@@ -1,11 +1,9 @@
 // ---------------------------------------------------------------------------
-// $Id: CmdLineParser.h 232 2008-08-15 18:19:35Z daaugusto $
-//
 //   CmdLineParser.h (created on Tue Aug 8 11:01:58 BRT 2006)
 // 
 //   Command-line Parser
 //
-//   Copyright (C) 2006-2008 Douglas Adriano Augusto (daaugusto)
+//   Copyright (C) 2006-2015 Douglas Adriano Augusto (daaugusto)
 // 
 // Command-line Parser is free software; you can redistribute it and/or
 // modify it under the terms of the GNU General Public License as published
@@ -291,6 +289,7 @@
 #include <limits>
 #include <map>
 #include <cstring>
+#include <iostream>
 
 #include "CmdLineException.h"
 
@@ -618,7 +617,7 @@ public:
    OptionsFLOAT( const Parser& parser ): OptionsType<FLOAT>( parser ) {}
 
    Option<FLOAT>& Add( const std::string&, const std::string& = "", 
-                       FLOAT = 0.0, FLOAT = std::numeric_limits<FLOAT>::min(), 
+                       FLOAT = 0.0, FLOAT = -std::numeric_limits<FLOAT>::max(), 
                        FLOAT = std::numeric_limits<FLOAT>::max() );
 
    /**
